"""Collector class used for defining all data collection points to Atom Finance."""
import jinja2

from pyatom_finance import config
from pyatom_finance.exceptions import StockCollectorRequesterError
from pyatom_finance.requester import Requester


class Collector:
    """Collector Class"""

    def __init__(self):
        self.requester = Requester()
        self.requester.create_session()
        loader = jinja2.FileSystemLoader(searchpath=config.SETTINGS.jinja2_templates)
        self.template_env = jinja2.Environment(loader=loader, autoescape=True)

    def _render_template(self, template, **kwargs):
        template = self.template_env.get_template(template)
        return template.render(**kwargs)

    def check_symbol_exists(self, symbol):
        """Ensure symbol exists before use."""
        payload = {
            "operationName": "getSymbol",
            "variables": {"symbol": symbol},
            "query": self._render_template("get_symbol.j2"),
        }
        resp = self.requester.post_request(payload)
        if not resp["data"]["symbol"]:
            raise StockCollectorRequesterError(
                "Unknown Symbol", f"Unable to find symbol in Atom, please check symbol: {symbol}"
            )
        return True

    def get_better_consensuses(self, symbol):
        """getBetterConsensuses"""
        payload = {
            "operationName": "getBetterConsensuses",
            "variables": {"symbol": symbol},
            "query": self._render_template("get_better_consensuses.j2"),
        }
        return self.requester.post_request(payload)

    def get_brokerage_actions(self, symbol):
        """getBrokerageActions"""
        payload = {
            "operationName": "getBrokerageActions",
            "variables": {"symbols": [symbol]},
            "query": self._render_template("get_brokerage_actions.j2"),
        }
        return self.requester.post_request(payload)

    def get_chart_interday(self, symbol):
        """getChartInterday"""
        payload = {
            "operationName": "getChartInterday",
            "variables": {"symbol": symbol},
            "query": self._render_template("get_chart_interday.j2"),
        }
        return self.requester.post_request(payload)

    def get_chart_intraday(self, symbol):
        """getChartIntraday"""
        payload = {
            "operationName": "getChartIntraday",
            "variables": {"symbol": symbol},
            "query": self._render_template("get_chart_intraday.j2"),
        }
        return self.requester.post_request(payload)

    def get_financial_data(self, symbol):
        """getFinancialData"""
        payload = {
            "operationName": "getFinancialData",
            "variables": {"symbol": symbol},
            "query": self._render_template("get_financial_data.j2"),
        }
        return self.requester.post_request(payload)

    def get_future_events(self, symbol):
        """getFutureEvents"""
        payload = {
            "operationName": "getManyEvents",
            "variables": {"symbols": [symbol]},
            "query": self._render_template("get_future_events.j2"),
        }
        return self.requester.post_request(payload)

    def get_market_cap(self, symbol):
        """getMarketCap"""
        payload = {
            "operationName": "getMarketCap",
            "variables": {"symbol": symbol},
            "query": self._render_template("get_market_cap.j2"),
        }
        return self.requester.post_request(payload)

    def get_many_events(self, symbol):
        """getManyEvents"""
        payload = {
            "operationName": "getManyEvents",
            "variables": {"symbols": [symbol]},
            "query": self._render_template("get_many_events.j2"),
        }
        return self.requester.post_request(payload)

    def get_near_events(self, symbol):
        """getNearEvents"""
        payload = {
            "operationName": "getNearEvents",
            "variables": {"symbols": [symbol]},
            "query": self._render_template("get_near_events.j2"),
        }
        return self.requester.post_request(payload)

    def get_news_feed(self, symbol):
        """getNewsFeed"""
        payload = {
            "operationName": "getNewsFeed",
            "variables": {
                "symbols": [symbol],
                "page": 0,
                "research": False,
            },
            "query": self._render_template("get_news_feed.j2"),
        }
        return self.requester.post_request(payload)

    def get_overview_data(self, symbol):
        """getOverviewData"""
        payload = {
            "operationName": "getOverviewData",
            "variables": {"symbol": symbol},
            "query": self._render_template("get_overview_data.j2"),
        }
        return self.requester.post_request(payload)

    def get_past_events(self, symbol):
        """getPastEvents"""
        payload = {
            "operationName": "getManyEvents",
            "variables": {"symbols": [symbol]},
            "query": self._render_template("get_past_events.j2"),
        }
        return self.requester.post_request(payload)

    def get_period_guidance(self, symbol):
        """getPeriodGuidance"""
        payload = {
            "operationName": "getPeriodGuidance",
            "variables": {
                "symbol": symbol,
                "start": 0,
                "end": 3,
            },
            "query": self._render_template("get_period_guidance.j2"),
        }
        return self.requester.post_request(payload)

    def get_relative_consensuses(self, symbol):
        """getRelativeConsensuses"""
        payload = {
            "operationName": "getRelativeConsensuses",
            "variables": {
                "symbol": symbol,
                "start": 0,
                "end": 3,
            },
            "query": self._render_template("get_relative_consensuses.j2"),
        }
        return self.requester.post_request(payload)

    def get_symbol(self, symbol):
        """getSymbol"""
        payload = {
            "operationName": "getSymbol",
            "variables": {"symbol": symbol},
            "query": self._render_template("get_symbol.j2"),
        }
        return self.requester.post_request(payload)



"""Custom exceptions"""


class StockCollectorRequesterError(Exception):
    """General exception for HTTP/HTTPS requests"""

    def __init__(self, reason, message, **kwargs):
        """Exception Init."""
        super(StockCollectorRequesterError, self).__init__(kwargs)
        self.reason = reason
        self.message = message

    def __str__(self):
        """Exception __str__."""
        return f"{self.__class__.__name__}: {self.reason}: {self.message}"


class AtomLoginError(Exception):
    """Atom login error exception"""

    def __init__(self, reason, message, **kwargs):
        """Exception Init."""
        super(AtomLoginError, self).__init__(kwargs)
        self.reason = reason
        self.message = message

    def __str__(self):
        """Exception __str__."""
        return f"{self.__class__.__name__}: {self.reason}: {self.message}"



"""Generic requests class"""
import json
import requests

from pyatom_finance import config
from pyatom_finance.exceptions import AtomLoginError, StockCollectorRequesterError


class Requester:
    """Requester class for saving Atom Finance session"""

    def __init__(self):
        self.verify = True
        self.headers = {"Content-Type": "application/json"}
        self.session = requests.session()

    def create_session(self):
        """Create session saving coookies for auth"""
        payload = {
            "username": config.SETTINGS.username,
            "password": config.SETTINGS.password,
        }
        resp = self.session.request(
            "POST",
            config.SETTINGS.atom_signin_url,
            data=json.dumps(payload),
            headers=self.headers,
        )
        if 199 < resp.status_code < 300:
            resp = resp.json()
            if not resp["success"]:
                raise AtomLoginError(reason="Unable to login to Atom Finance", message=resp["error"])
        else:
            raise StockCollectorRequesterError(reason="Bad HTTP statuss code", message=resp.text)

    def post_request(self, query):
        """Generic post request"""
        resp = self.session.request(
            "POST",
            config.SETTINGS.atom_url,
            data=json.dumps(query),
            headers=self.headers,
        )
        if 199 < resp.status_code < 300:
            return resp.json()
        raise StockCollectorRequesterError(reason="Bad HTTP status code", message=resp.text)



"""PyDantic global settings and configuration"""
import os

from pathlib import Path
from pydantic import BaseSettings

import toml


SETTINGS = None


class Settings(BaseSettings):
    """Base settings used throughout PyAtom_Finance"""

    atom_url: str = "https://atom.finance/graphql"
    atom_signin_url: str = "https://atom.finance/session/signin"
    base_dir: str = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    jinja2_templates: str = os.path.join(base_dir, "pyatom_finance/templates")
    username: str = os.environ.get("ATOM_USERNAME", "")
    password: str = os.environ.get("ATOM_PASSWORD", "")


def load(config_file_name="pyproject.toml"):
    """PyDantic load function"""
    global SETTINGS
    if os.path.exists(config_file_name):
        config_string = Path(config_file_name).read_text()
        config_tmp = toml.loads(config_string)

        if "tool" in config_tmp and "atom_finance" in config_tmp.get("tool", {}):
            SETTINGS = Settings(**config_tmp["tool"]["atom_finance"])
            return

    SETTINGS = Settings()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyatom_finance']

package_data = \
{'': ['*'], 'pyatom_finance': ['templates/*']}

install_requires = \
['Jinja2>=2.11.3,<3.0.0', 'pydantic>=1.8.1,<2.0.0', 'requests>=2.25.1,<3.0.0']

setup_kwargs = {
    'name': 'pyatom-finance',
    'version': '0.1.0',
    'description': 'Module to collect data from Atom IO',
    'long_description': None,
    'author': 'carbonarok',
    'author_email': 'carbonarok@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
